--[=[
Recruiter World of Warcraft Add-on
Copyright (c) 2015 by Eric Isakson (eric.david.isakson@gmail.com), All rights reserved

Version r20151123034449

Recruiter is a tool for coordinating recruitment activities accross guildmates.
It is intended to help provide a consistent message in recruiting activities and
prevent spamming community members with invitations. It is also useful
for managing guild membership with tools for removing inactive members and 
banning players so they are not re-invited by a recruiter after being kicked
from the guild.

This add-on uses the Ace3 framework.

type /recruiter to get a list of existing options.

-----
    Announcer.lua
-----

This component plays alert sounds and display messages.


--]=]

local ERROR     = 1;
local WARNING   = 2;
local INFO      = 3;
local INFO2     = 4;


local ADDON_NAME, T = ...;
local RECRUITER_ = T.Recruiter;
local L = RECRUITER_.Localized_Text;

-- Create module
RECRUITER_.Announcer = RECRUITER_:NewModule("Announcer", "AceConsole-3.0");
local Announcer = RECRUITER_.Announcer;

-- Up Values
local UnitGUID      = _G.UnitGUID;
local UnitName      = _G.UnitName;
local UnitClass     = _G.UnitClass;
local UnitSex       = _G.UnitSex;
local PlaySoundFile = _G.PlaySoundFile;
local select        = _G.select;

function Announcer:OnInitialize() -- {{{
    self:Debug(INFO, "OnInitialize called!");
    self.db = RECRUITER_.db:RegisterNamespace('Announcer', {
        global = {
            ChatMessages = false,
            Sounds = true,

            PostToChat = false,
            PostToChatThrottle = 2 * 60,
            ContactingMessage = false,
            OpeningMessage = false,
            PostChannel = 'AUTO',
        },
    });

    

end -- }}}

function Announcer:GetOptions () -- {{{


    local validatePostChatMessage = function (info, v)

        local counterpartMessage = info[#info] == 'OpeningMessage' and 'ContactingMessage' or 'OpeningMessage';
        Announcer:Debug(INFO, 'counterpartMessage:', counterpartMessage);
      
        if not v:find('%[LEAD%]') then
            return self:Error(L["OPT_POST_ANNOUNCE_MISSING_KEYWORD"]);
        end

        if v:len() < ("%[LEAD%]"):len() + 10 then
            return self:Error(L["OPT_POST_ANNOUNCE_MESSAGE_TOO_SHORT"]);
        end

        if v == Announcer.db.global[counterpartMessage] then
            return self:Error(L["OPT_POST_ANNOUNCE_MESSAGES_EQUAL"]);
        end

        return 0, v;

    end

    return {
        [Announcer:GetName()] = {
            name = L[Announcer:GetName()],
            type = 'group',
            get = function (info) return Announcer.db.global[info[#info]]; end,
            set = function (info, value) RECRUITER_:SetHandler(self, info, value) end,
            args = {
                ChatMessages = {
                    type = 'toggle',
                    name = L["OPT_ANNOUNCE"],
                    desc = L["OPT_ANNOUNCE_DESC"],
                    order = 1,
                },
                Sounds = {
                    type = 'toggle',
                    name = L["OPT_SOUNDS"],
                    desc = L["OPT_SOUNDS_DESC"],
                    order = 10,
                },
                -- enable
                PostToChat = {
                    type = 'toggle',
                    name = L["OPT_POST_ANNOUNCE_ENABLE"],
                    desc = L["OPT_POST_ANNOUNCE_ENABLE_DESC"],
                    set = function (info, v)
                        Announcer.db.global.PostToChat = v;
                    end,
                    order = 30,
                },
                PostToChatOptions = {
                    type = 'group',
                    inline = true,
                    name = L['OPT_POST_ANNOUNCE_SETTINGS'],
                    hidden = function() return Announcer.db.global.PostToChat == false end,
                    order = 40,
                    args = {
                        Description = {
                            type = 'description',
                            order = 0,
                            name = L["OPT_POST_ANNOUNCE_DESCRIPTION"],
                        },
                        PostChannel = {
                            type = 'select',
                            name = L['OPT_POST_ANNOUNCE_CHANNEL'],
                            desc = L['OPT_POST_ANNOUNCE_CHANNEL_DESC'],
                            values = { ['AUTO'] = L['AUTO_OFFICER_GUILD'], ['OFFICER'] = L['OFFICER'], ['GUILD'] = L['GUILD']},
                            order = 30,
                        },
                        -- throttle
                        PostToChatThrottle = {
                            type = 'range',
                            min = 60,
                            max = 10 * 60,
                            step = 1,
                            bigStep = 5,
                            name = L["OPT_POST_ANNOUNCE_THROTTLE"],
                            desc = L["OPT_POST_ANNOUNCE_THROTTLE_DESC"],
                            order = 40,
                        },
                        ValidityCheck = {
                            type = 'description',
                            order = 45,
                            name = RECRUITER_:ColorText(L["OPT_POST_ANNOUNCE_POST_MESSAGE_ISSUE"],'FFFF4040'),
                            hidden = function ()
                                if Announcer.db.global.PostToChat == false
                                    or (Announcer.db.global.ContactingMessage and Announcer.db.global.OpeningMessage) then

                                    return true;
                                else
                                    return false;
                                end
                            end
                        },
                        OpeningMessage = {
                            type = 'input',
                            width = 'full',
                            name = L["OPT_POST_ANNOUNCE_OPENING_MESSAGE"],
                            desc = L["OPT_POST_ANNOUNCE_OPENING_MESSAGE_DESC"],
                            get = function (info)
                                return Announcer.db.global[info[#info]] or L["OPT_POST_ANNOUNCE_OPENING_MESSAGE_DEFAULT"];
                            end,
                            validate = validatePostChatMessage,
                            order = 50,
                        },
                        ContactingMessage = {
                            type = 'input',
                            width = 'full',
                            name = L["OPT_POST_ANNOUNCE_CONTACTING_MESSAGE"],
                            desc = L["OPT_POST_ANNOUNCE_CONTACTING_MESSAGE_DESC"],
                            get = function (info)
                                return Announcer.db.global[info[#info]] or L["OPT_POST_ANNOUNCE_CONTACTING_MESSAGE_DEFAULT"];
                            end,
                            validate = validatePostChatMessage,
                            order = 60,
                        },
                    },
                },
            },
        },
    };
end -- }}}


function Announcer:OnEnable() -- {{{
    self:Debug(INFO, "OnEnable");

    -- Subscribe to RECRUITER_ callbacks
    self:RegisterMessage("RECRUITER_LEAD_MOUSE_OVER");
    self:RegisterMessage("RECRUITER_LEAD_TARGETTED");

    self:RegisterChatCommand("recruiterp", function() self:ChatPlacard() end);

end -- }}}

function Announcer:OnDisable() -- {{{
    self:Debug(INFO2, "OnDisable");
    self:UnregisterChatCommand("recruiterp");
end -- }}}


-- Internal CallBacks (RECRUITER_LEAD_MOUSE_OVER - RECRUITER_LEAD_TARGETTED) {{{
local previousUnitGuid;
function Announcer:RECRUITER_LEAD_MOUSE_OVER(selfevent, leadProfile)

    if previousUnitGuid ~= leadProfile.guid then
        self:RecruitThatLead(leadProfile, "Sound\\interface\\AlarmClockWarning3.ogg");
        previousUnitGuid = leadProfile.guid;
    end
end

function Announcer:RECRUITER_LEAD_TARGETTED (selfevent, leadProfile)
    self:RecruitThatLead(leadProfile, "Sound\\interface\\AuctionWindowOpen.ogg");
end

-- }}}


function Announcer:RecruitThatLead(leadProfile, soundFile)
    self:PlaySoundFile("Sound\\interface\\AuctionWindowOpen.ogg");
    local subjectColor = RECRUITER_:GetClassHexColor(leadProfile.class);
    self:Announce(L["RECRUIT_THAT_LEAD"]:format("|c" .. subjectColor, leadProfile.name));
end

function Announcer:Announce(...) -- {{{
    if self.db.global.ChatMessages then
        RECRUITER_:Print(...);
    end
end -- }}}

do

    local function GetDistributionChannel()
        local channel = Announcer.db.global.PostChannel;

        if channel ~= 'AUTO' then
            return channel;
        end

        -- if we are allowed to speak in officer chat
        -- TODO how do we check this and what channel name to return
--        if GetNumGroupMembers(LE_PARTY_CATEGORY_INSTANCE) > 0 then
--            return "INSTANCE_CHAT";
--        end

        -- if we are allowed to speak in guild chat
        -- TODO how do we check this and what channel name to return

        return "WHISPER";
    end

    local function Post(text)
        local channelType = GetDistributionChannel();

        --  SendChatMessage("msg" [,"type" [,"lang" [,"channel"] ] ]).
        SendChatMessage("RECRUITER_: " .. text, channelType, nil, channelType == 'WHISPER' and (UnitName('player')) or nil);
    end

    local LastAnnounce = 0;
    function Announcer:ChatPlacard()
        -- first check config
        if not (self.db.global.PostToChat and self.db.global.ContactingMessage and self.db.global.OpeningMessage) then
            self:Error(L["CHAT_POST_ANNOUNCE_FEATURE_NOT_CONFIGURED"]);
            return false;
        end
        local config = self.db.global;

        -- then check throttle
        if GetTime() - LastAnnounce < config.PostToChatThrottle then
            self:Error(L["CHAT_POST_ANNOUNCE_TOO_SOON_WAIT"]);
            return false;
        end

        -- send to chat
        local lead = "Leafrik"; -- TODO get the lead correctly
        local guild = "TODO_GUILD"; -- TODO get the lead correctly
        local RecruitersText = ( config.ContactingMessage:gsub('%[LEAD%]', lead) );
        self:Debug(INFO, "RECRUITER_:", RecruitersText);
        Post(RecruitersText);
        local OpeningText = ( config.OpeningMessage:gsub('%[LEAD%]', lead):gsub('%[GUILD%]', guild) );
        self:Debug(INFO, "RECRUITER_:", OpeningText);
        SendChatMessage(OpeningText, "WHISPER", nil, lead);
        -- log the time to prevent spam
        LastAnnounce = GetTime();

        return true;
    end
end

function Announcer:PlaySoundFile(...) -- {{{
    if self.db.global.Sounds then
        PlaySoundFile(...);
    end
end -- }}}
