--[=[
Recruiter World of Warcraft Add-on
Copyright (c) 2015 by Eric Isakson (eric.david.isakson@gmail.com), All rights reserved

Version r20151123034449

Recruiter is a tool for coordinating recruitment activities accross guildmates.
It is intended to help provide a consistent message in recruiting activities and
prevent spamming community members with invitations. It is also useful
for managing guild membership with tools for removing inactive members and 
banning players so they are not re-invited by a recruiter after being kicked
from the guild.

This add-on uses the Ace3 framework.

type /recruiter to get a list of existing options.

-----
    Core.lua
-----


--]=]

--========= NAMING Convention ==========
--      VARIABLES AND FUNCTIONS (upvalues excluded)
-- global variable                == _NAME_WORD2 (underscore + full uppercase)
-- semi-global (file locals)      == NAME_WORD2 (full uppercase)
-- locals to closures or members  == NameWord2
-- locals to functions            == nameWord2
--
--      TABLES
--  globals                       == NAME__WORD2
--  locals                        == name_word2
--  members                       == Name_Word2

-- Debug templates
local ERROR     = 1;
local WARNING   = 2;
local INFO      = 3;
local INFO2     = 4;

local UNPACKAGED = "@pro" .. "ject-version@";
local VERSION = "r20151123034449";

local ADDON_NAME, T = ...;

T._FatalError = function (TheError)

    if not StaticPopupDialogs["RECRUITER_ERROR_FRAME"] then
        StaticPopupDialogs["RECRUITER_ERROR_FRAME"] = {
            text = "|cFFFF0000Recruiter Fatal Error:|r\n%s",
            button1 = "OK",
            OnAccept = function()
                T._FatalError_Diaplayed = false;
                return false;
            end,
            timeout = 0,
            whileDead = 1,
            hideOnEscape = 1,
            showAlert = 1,
            preferredIndex = 3,
        };
    end

    if not T._FatalError_Diaplayed then
        StaticPopup_Show ("RECRUITER_ERROR_FRAME", TheError);
        if T._DiagStatus then
            T._FatalError_Diaplayed = true;
        end
    end
end

local _, _, _, tocversion = GetBuildInfo();
T._tocversion = tocversion;

-- === Add-on basics and variable declarations {{{
T.Recruiter = LibStub("AceAddon-3.0"):NewAddon("Recruiter", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0");
local RECRUITER = T.Recruiter;

--[===[@debug@
_RECRUITER_DEBUG = RECRUITER;
--@end-debug@]===]

RECRUITER.Localized_Text = LibStub("AceLocale-3.0"):GetLocale("Recruiter", true);

local L = RECRUITER.Localized_Text;

RECRUITER.Constants = {};
local RECRUITER_C = RECRUITER.Constants;

RECRUITER_C.MaxTOC = tonumber(GetAddOnMetadata("Recruiter", "X-Max-Interface") or math.huge);

-- The header for RECRUITER key bindings
BINDING_HEADER_RECRUITER = "Recruiter";
BINDING_NAME_RECRUITERP = L["OPT_POST_ANNOUNCE_ENABLE"];

RECRUITER.DelayedFunctionCallsCount  = 0;
RECRUITER.DelayedFunctionCalls       = {};

--[=[
local lead_template = {
    guid        = "",
    name        = "unknown",
    realm       = "",
    class       = "unknown",
    level       = 0,
    zone        = "unknown",
};
--]=]

RECRUITER.Registry_by_GUID = {}; -- [guid] = lead_template

-- upvalues {{{
local _G                = _G;
local UnitGUID          = _G.UnitGUID;
local UnitIsUnit        = _G.UnitIsUnit;
local UnitClass         = _G.UnitClass;
local UnitName          = _G.UnitName;
local UnitLevel         = _G.UnitLevel;
local select            = _G.select;

-- }}}

-- Modules standards configurations {{{

-- Configure default libraries for modules
RECRUITER:SetDefaultModuleLibraries( "AceConsole-3.0", "AceEvent-3.0")

-- Set the default prototype for modules
RECRUITER.MODULE_PROTOTYPE   = {
    OnEnable = function(self) self:Debug(INFO, "prototype OnEnable called!") end,

    OnDisable = function(self) self:Debug(INFO, "prototype OnDisable called!") end,

    OnInitialize = function(self)
        self:Debug(INFO, "prototype OnInitialize called!");
    end,

    Debug = function(self, ...) RECRUITER.Debug(self, ...) end,

    Error = function(self, m) RECRUITER.Error (self, m) end,

}

RECRUITER:SetDefaultModulePrototype( RECRUITER.MODULE_PROTOTYPE )

-- Set modules' default state to "false"
RECRUITER:SetDefaultModuleState( false )
-- }}}

-- modules handling functions {{{

function RECRUITER:SetModulesStates ()
    for moduleName, module in self:IterateModules() do
        self:Debug(INFO2, 'enabling module:', moduleName);
        module:SetEnabledState(self.db.global.Modules[moduleName].Enabled);
    end
end

-- }}}

-- == Options and defaults {{{
do

    local function GetCoreOptions() -- {{{
    return {
        type = 'group',
        get = function (info) return RECRUITER.db.global[info[#info]]; end,
        set = function (info, value) RECRUITER:SetHandler(RECRUITER, info, value) end,
        disabled = function () return not RECRUITER:IsEnabled(); end,
        childGroups = 'tab',
        name = "Recruiter",
        args = {
            Description = {
                type = 'description',
                name = L["DESCRIPTION"],
                order = 0,
            },
            On = {
                type = 'toggle',
                name = L["OPT_ON"],
                desc = L["OPT_ON_DESC"],
                set = function(info) RECRUITER.db.global.Enabled = true; RECRUITER:Enable(); return RECRUITER.db.global.Enabled; end,
                get = function(info) return RECRUITER:IsEnabled(); end,
                hidden = function() return RECRUITER:IsEnabled(); end, 

                disabled = false,
                order = 1,
            },
            Off = {
                type = 'toggle',
                name = L["OPT_OFF"],
                desc = L["OPT_OFF_DESC"],
                set = function(info) RECRUITER.db.global.Enabled = not RECRUITER:Disable(); return not RECRUITER.db.global.Enabled; end,
                get = function(info) return not RECRUITER:IsEnabled(); end,
                guiHidden = true,
                hidden = function() return not RECRUITER:IsEnabled(); end, 
                order = -1,
            },
            Debug = {
                type = 'toggle',
                name = L["OPT_DEBUG"],
                desc = L["OPT_DEBUG_DESC"],
                guiHidden = true,
                disabled = false,
                order = -2,
            },
            DebugLevel = {
                type = 'range',
                name = L["OPT_DEBUGLEVEL"],
                desc = L["OPT_DEBUGLEVEL_DESC"],
                min = 1,
                max = 3,
                guiHidden = true,
                disabled = false,
                order = -3,
            },
            
            Version = {
                type = 'execute',
                name = L["OPT_VERSION"],
                desc = L["OPT_VERSION_DESC"],
                guiHidden = true,
                func = function () RECRUITER:Print(L["VERSION"], 'r20151123034449,', L["RELEASE_DATE"], '2015-11-23T03:44:49Z') end,
                order = -5,
            },
            core = {
                type = 'group',
                name =  L["OPT_CORE_OPTIONS"],
                order = 1,
                args = {
                    Info_Header = {
                        type = 'header',
                        name = L["VERSION"] .. ' r20151123034449 -- ' .. L["RELEASE_DATE"] .. ' 2015-11-23T03:44:49Z',
                        order = 1,
                    },
                    testOnTarget = {
                        type = 'execute',
                        name = L["OPT_TESTONTARGET"],
                        desc = L["OPT_TESTONTARGET_DESC"],
                        func = function ()
                            if (UnitGUID("target")) then
                                RECRUITER:MakeDummyEvent("target");
                            else
                                RECRUITER:Print( L["OPT_TESTONTARGET_ENOTARGET"] );
                            end

                        end,
                        order = 350,
                    },
                    Modules = {
                        type = 'group',
                        name = L["OPT_MODULES"],
                        inline = true,
                        handler = {
                            ["hidden"]   = function () return not RECRUITER:IsEnabled(); end,
                            ["disabled"] = function () return not RECRUITER:IsEnabled(); end,

                            ["get"] = function (handler, info) return (RECRUITER:GetModule(info[#info])):IsEnabled(); end,
                            ["set"] = function (handler, info, value) 

                                RECRUITER.db.global.Modules[info[#info]].Enabled = value;
                                local result;

                                if value then
                                    result = RECRUITER:EnableModule(info[#info]);
                                    if result then
                                        RECRUITER:Print(info[#info], RECRUITER:ColorText(L["OPT_ON"], "FF00FF00"));
                                    end
                                else
                                    result = RECRUITER:DisableModule(info[#info]);
                                    if result then
                                        RECRUITER:Print(info[#info], RECRUITER:ColorText(L["OPT_OFF"], "FFFF0000"));
                                    end
                                end

                                return result;
                            end,
                        },
                        -- Enable-modules-check-boxes (filled by modules)
                        args = {},
                        order = 900,
                    },
                    Header1 = {
                        type = 'header',
                        name = '',
                        order = 400,
                    },
                    LeadForgetDuration = {
                        type = "range",
                        name = L["OPT_LEAD_FORGET_DURATION"],
                        desc = L["OPT_LEAD_FORGET_DURATION_DESC"],
                        min = 7,
                        max = 4 * 7, -- forget after 4 weeks
                        step = 1,
                        bigStep = 7,
                        order = 500,
                    },
                    UpdateLeadsInterval = {
                        type = "range",
                        name = L["OPT_UPDATE_LEADS_INTERVAL"],
                        desc = L["OPT_UPDATE_LEADS_INTERVAL_DESC"],
                        min = 60,
                        max = 10 * 60, -- 10 minutes
                        step = 15,
                        bigStep = 60,
                        order = 600,
                    },
                    Header1000 = {
                        type = 'header',
                        name = '',
                        order = 999,
                    },
                },
            },
        },
    };
    end -- }}}

    -- Used in Ace3 option table to get feedback when setting options through command line
    function RECRUITER:SetHandler (module, info, value)

        module.db.global[info[#info]] = value;

        if info["uiType"] == "cmd" then

            if value == true then
                value = L["OPT_ON"];
            elseif value == false then
                value = L["OPT_OFF"];
            end

            self:Print(RECRUITER:ColorText(RECRUITER:GetOPtionPath(info), "FF00DD00"), "=>", RECRUITER:ColorText(value, "FF3399EE"));
        end
    end
    

    local Enable_Module_CheckBox = {
        type = 'toggle',
        name = function (info) return L[info[#info]] end, -- it should be the localized module name
        desc = function (info) return L[info[#info] .. "_DESC"] end, 
        get = "get",
        set = "set",
        disabled = "disabled",
    };

    -- get the option tables feeding it with the core options and adding modules options
    function RECRUITER.GetOptions()
        local options = GetCoreOptions();

        -- Add modules enable/disable checkboxes
        for moduleName, module in RECRUITER:IterateModules() do
            if not options.args.core.args.Modules.args[moduleName] then
                options.args.core.args.Modules.args[moduleName] = Enable_Module_CheckBox;
            else
                error("RECRUITER: module name collision!");
            end
            -- Add modules specific options
            if module.GetOptions then
                if module:IsEnabled() then
                    if not options.plugins then options.plugins = {} end;
                    options.plugins[moduleName] = module:GetOptions();
                end
            end
        end

        return options;
    end
end


local DEFAULT__CONFIGURATION = {
    global = {
        Modules = {
            ['**'] = {
                Enabled = true, -- Modules are enabled by default
            },
        },
        LeadForgetDuration = 28, -- 28 days
        UpdateLeadsInterval = 10 * 60; -- 10 minutes
        Enabled = true,
        Debug = false,
        DebugLevel = 1,
        --@alpha@
        Debug = true,
        DebugLevel = 2,
        --@end-alpha@
        Log = false,
    },
};
-- }}}

-- = Add-on Management functions {{{
function RECRUITER:OnInitialize()
-- Catch people updating add-ons while WoW is running before they post "it doesn't work!!!!" comments.
    local versionInTOC = GetAddOnMetadata("Recruiter", "Version");

    if versionInTOC and versionInTOC ~= VERSION and versionInTOC ~= UNPACKAGED and VERSION ~= UNPACKAGED then
        T._DiagStatus = 2;
        T._Diagmessage = "You have updated Recruiter while WoW was still running in the background.\n\nYou need to restart WoW completely or you might experience various issues with your add-ons until you do.";
        T._FatalError(T._Diagmessage);
    end

    self.db = LibStub("AceDB-3.0"):New("Recruiter", DEFAULT__CONFIGURATION);

    LibStub("AceConfig-3.0"):RegisterOptionsTable(tostring(self), self.GetOptions, {"Recruiter", "recruiter"});
    
    self:RegisterChatCommand('recruiterg', function() LibStub("AceConfigDialog-3.0"):Open(tostring(self)) end, true);

    self:CreateClassColorTables();

    self:SetEnabledState(self.db.global.Enabled);

end

function RECRUITER:OnEnable()

    if T._DiagStatus == 2 then
        self:Disable();
        return;
    end

    self:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "TestUnit");
    self:RegisterEvent("PLAYER_TARGET_CHANGED", "TestUnit");
    
    self:Print(L["ENABLED"]);

    self:SetModulesStates();

    self:ScheduleRepeatingTimer(self.UpdateLeads, self.db.global.UpdateLeadsInterval, self);

end

function RECRUITER:OnDisable()

    self:Print(L["DISABLED"]);

    if T._DiagStatus == 2 then
        self:Print("|cFFD00000"..T._Diagmessage.."|r");
    end

end
-- }}}

function RECRUITER:UpdateLeads()
    -- TODO figure out the leads using /who data
    local guid = UnitGUID("player");
    if self.Registry_by_GUID[guid] == nil then
        local leadProfile = {
            guid        = guid,
            name        = select(1,UnitName("player")),
            class       = select(2,UnitClass("player")),
            level       = UnitLevel("player"),
            zone        = "unknown",
        };
        self.Registry_by_GUID[guid] = leadProfile;
    end
    return self.Registry_by_GUID
end

-- MouseOver and Target trigger {{{
do
    local LastDetectedGUID = "";
    function RECRUITER:TestUnit(eventName) -- XXX to check/rewrite

        local unit="";
        if eventName=="UPDATE_MOUSEOVER_UNIT" then
            unit = "mouseover";
        elseif eventName=="PLAYER_TARGET_CHANGED" then
            unit = "target";
        else
            self:Print("called on invalid event");
            return;
        end

        local unitGuid = UnitGUID(unit);
        if not unitGuid then
            --self:Debug(WARNING, "No unit GUID");
            return;
        end

        if RECRUITER.Registry_by_GUID[unitGuid] then

            if LastDetectedGUID == unitGuid and unit == "target" then
                self:SendMessage("RECRUITER_LEAD_TARGETTED", RECRUITER.Registry_by_GUID[unitGuid]);
                return;
            end

            if not UnitIsUnit("mouseover", "target") and unit == "mouseover" then
                self:SendMessage("RECRUITER_LEAD_MOUSE_OVER", RECRUITER.Registry_by_GUID[unitGuid]);
                LastDetectedGUID = unitGuid;
            end
        end
        
    end
end -- }}}

